!  x---------------------------------------------------------------x
!  |  Global variables used in PMF program                         |
!  x---------------------------------------------------------------x

   module pmf_parm

   implicit none

   save

   integer, parameter :: io_unit   = 25 &                  ! Unit for reading EVB data
                       , pmf_unit  = 50 &                  ! Unit for outputing PMF data
                       , bias_size = 10 &                  ! Maximum # of biasing potential 
                       , file_size = 200 &                 ! Maximum # of biased sampling data files
                       , nwin_size = file_size &           ! Maximum # of biased sampling windows
                       , slice_size = 200 &                ! Maximum # of data slices in a 
                                                           ! concatenated data file
                       , data_type_size = 50               ! Types of data in data file

   character(512), parameter :: wham_ifile = 'PMF.INPUT'   ! Input file for PMF program

   real*8, parameter :: mgc = 8.314472d-3                  ! Molar gas constant in units of kj/mol/K (CODATA)

   real*8, parameter :: joule2cal = 1.0d0 / 4.184d0        ! Conversion factor (NIST)

   real*8 :: data_tol                                      ! For comparing reconstructed data to those 
                                                           !   output from Amber
   integer :: nevb &                                       ! Number of EVB states
            , nbias &                                      ! Number of biasing potential/per window
            , nwin &                                       ! Number of biased sampling window
            , nbead &                                      ! Number of beads in EVB/PIMD
            , ndata_max &                                  ! Maximum number of data points over all windows
            , RC_data_max &                                ! Maximum number of RC data points over all windows
            , evb_step &                                   ! Interval between each output of EVB data
            , num_bin &                                    ! Number of RC bins
            , neject_1stdata &                             ! Eject these beginning data points 
            , neject_highumb &                             ! Eject these highest umbrella data points
            , neject_lowumb                                ! Eject these lowest umbrealla data points

   integer, allocatable :: ndata(:)                        ! (NWIN): Number of data points for window

   integer, allocatable :: RC_data(:)                      ! (NWIN): Number of RC data points for window

   real*8 :: bin_start &                                   ! Starting value of RC
           , bin_width, bin_width_inv &                    ! Bin width and inverse
           , beta, beta_inv &                              ! 1/(k_B T) and inverse
           , dt &                                          ! MD timestep size
           , temperature &                                 ! Temperature in Kelvin
           , hits_tol                                      ! Minimum number of hits tolerance

   real*8, allocatable :: rcrd(:)                          ! (NUM_BIN): reaction coordinate

   logical :: debug &                                      ! Enable debuggin output
            , Lchk_data_slices &                           ! Check the consistency of concatenated data slices
            , f_k_thermint                                 ! Estimate FE constants from TI 
                                                           ! (for mapping potential/WHAM)

   character(512) :: dyn_type                              ! Dynamics type

   end module pmf_parm 

!  x---------------------------------------------------------------x
!  |  Mapping potential                                            |
!  x---------------------------------------------------------------x

   module map_pot 

   implicit none

   save

   type map_dat_type                                       ! Mapping potential derived type
      integer, pointer :: map_init(:), map_final(:)        ! (NBIAS): Initial state and final state
      real*8,  pointer :: lambda(:)                        ! (NBIAS): Mapping potential parameter
      real*8,  pointer :: evb_mat(:,:,:)                   ! (NEVB,NEVB,NDATA_MAX): EVB matrix
      character(512)   :: evb_file                         ! EVB data file name
   endtype

   type( map_dat_type ), allocatable :: map_dat    (:)     ! (NWIN): pruned mapping potential data
   type( map_dat_type ), allocatable :: map_dat_raw(:)     ! (NWIN): raw mapping potential data

   type awarshel_type                                      ! Ariel Warshel derived type
      real*8, pointer :: nrg_evb(:)                        ! (NDATA_MAX): EVB energy
      real*8, pointer :: nrg_map(:)                        ! (NDATA_MAX): mapping potential energy
      real*8, pointer :: nrg_gap(:)                        ! (NDATA_MAX): energy gap (used as RC)
      real*8, pointer :: dVmapdlam(:)                      ! (NDATA_MAX): (d / d lambda) V_map
      real*8          :: FE_map                            ! FE of mapping potential
      real*8          :: dVdl_avg                          ! < (d / d lambda V_map >
      real*8          :: dVdl_sd                           ! Standard deviation of (d / d lambda) V_map
   endtype

   type( awarshel_type ), allocatable :: awarshel    (:)   ! (NWIN): pruned Warshel data
   type( awarshel_type ), allocatable :: awarshel_raw(:)   ! (NWIN): raw Warshel data

   real*8, allocatable :: wpmf(:) &                        ! (NUM_BIN): Weighted PMF
                        , wpmf_print(:)                    ! (NUM_BIN): = 1.0 --> print data point

   type pmf_type 
      integer, pointer :: pb(:)                            ! (NUM_BIN): Biased distribution
      real*8,  pointer :: FE_el0(:)                        ! (NUM_BIN): FE on EVB ground state surface
      real*8,  pointer :: FE_map(:)                        ! (NUM_BIN): FE on mapping potential surface
      real*8,  pointer :: FE_ids(:)                        ! (NUM_BIN): FE on initial diabatic state
      real*8,  pointer :: FE_fds(:)                        ! (NUM_BIN): FE on final diabatic state
      real*8,  pointer :: dE0(:)                           ! (NUM_BIN): < exp[-beta*(V_el0-V_map)] >
      real*8,  pointer :: dEi(:)                           ! (NUM_BIN): < exp[-beta*( V_ii-V_map)] >
      real*8,  pointer :: dEf(:)                           ! (NUM_BIN): < exp[-beta*( V_ff-V_map)] >
   endtype

   type( pmf_type ), allocatable :: pmf(:)                 ! (NWIN): PMF segments

   end module map_pot 


!  x---------------------------------------------------------------x
!  |  Weighted Histogram Analysis Method                           |
!  x---------------------------------------------------------------x

   module wham_umb

   implicit none

   save

   real*8 :: wham_tol                                      ! WHAM convergence parameter

   real*8, allocatable :: wham_print(:)                    ! (NUM_BIN): = 1.0 --> print data point

   real*8, allocatable :: umb_pot    (:,:) &               ! (NDATA_MAX,NWIN): pruned V_umb= 0.5*k_umb*(RC-RC_0)^2
                        , umb_pot_raw(:,:)                 ! (NDATA_MAX,NWIN): raw

   real*8, allocatable :: V_tot    (:,:) &                 ! (NDATA_MAX,NWIN): pruned V_tot = V_0 + V_umb
                        , V_tot_raw(:,:)                   ! (NDATA_MAX,NWIN): raw V_tot

  real*8, allocatable :: vel0    (:,:) &                   ! (NDATA_MAX,NWIN): pruned Vel0
                       , vel0_raw(:,:)                     ! (NDATA_MAX,NWIN): raw Vel0

  real*8, allocatable :: rcrd_win    (:,:) &               ! (NDATA_MAX,NWIN): pruned nrg_gap RC
                       , rcrd_win_raw(:,:)                 ! (NDATA_MAX,NWIN): raw rcrd_win

   real*8, allocatable :: ebweta(:,:)                      ! (NUM_BIN,NWIN): exp(-beta V_umb) along RC

   real*8, allocatable :: ebfn(:)                          ! (NWIN): N_n * exp(+beta F_n)

   real*8, allocatable :: pb(:,:)                          ! (NUM_BIN,NWIN): Biased distribution

   real*8, allocatable :: pub(:)                           ! (NUM_BIN): Unbiased distribution

   real*8, allocatable :: f_k(:)                           ! (NWIN): FE constants 
 
   real*8, allocatable :: k_umb(:)                         ! (NWIN): Biasing potential harmonic force constant

   real*8, allocatable :: r0_umb(:)                        ! (NWIN): Equilibrium point for harmonic 
                                                           ! biasing potential

   type atm_index
      integer :: iatom                   ! atom index
      integer :: jatom                   ! atom index
      integer :: katom                   ! atom index
   endtype

   type( atm_index ), allocatable :: dbonds_RC(:,:)

   type( atm_index ), allocatable :: bond_RC(:,:)

   real*8, allocatable :: RC_raw(:,:,:) 

   end module wham_umb


!  x---------------------------------------------------------------x
!  |  PIMD                                                         |
!  x---------------------------------------------------------------x

   module pimd_dat

   implicit none

   save

   real*8, allocatable :: pimd_etot(:,:)                   ! (NDATA_MAX/NBEAD,NWIN) total energy

   real*8, allocatable :: pimd_ekin(:,:)                   ! (NDATA_MAX/NBEAD,NWIN) kinetic energy

   real*8, allocatable :: pimd_epot(:,:)                   ! (NDATA_MAX/NBEAD,NWIN) potential energy

   real*8, allocatable :: vel0_etot(:,:)                   ! (NDATA_MAX/NBEAD,NWIN) EVB ground-state total energy

   real*8, allocatable :: vel0_ekin(:,:)                   ! (NDATA_MAX/NBEAD,NWIN) EVB ground-state kinetic energy

   real*8, allocatable :: vel0_epot(:,:)                   ! (NDATA_MAX/NBEAD,NWIN) EVB ground-state potential energy

   end module pimd_dat




